#
# spec file for package gasnet
#
# Copyright (c) 2016 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


%define _buildshell /bin/bash
Name:           gasnet
Version:        1.28.2
%define libver	1_28_2
Release:        3.5
Summary:        A Communication Layer for GAS Languages
License:        PostgreSQL
Group:          Productivity/Networking/Other
Url:            https://bitbucket.org/berkeleylab/gasnet/
Source0:        https://bitbucket.org/berkeleylab/gasnet/downloads/GASNet-%{version}.tar.gz
BuildRequires:  automake
BuildRequires:  fdupes
BuildRequires:  gcc-c++
# For transition to rdma-core make sure the new packages are selected
# Once the transition is made the version check can be removed
BuildRequires:  libibverbs-devel >= 12
%ifarch x86_64
BuildRequires:  libfabric-devel
BuildRequires:  libpsm2-compat
BuildRequires:  libpsm2-devel
%endif
BuildRequires:  libtool
BuildRequires:  openmpi
BuildRequires:  openmpi-devel
ExcludeArch  :  s390 s390x

BuildRoot:      %{_tmppath}/%{name}-%{version}-build

%description
GASNet (Global Address Space Networking) is a language-independent,
low-level networking layer that provides network-independent
communication primitives tailored for implementing parallel global
address space SPMD languages such as UPC, Titanium, and Co-Array
Fortran.

%package devel
Summary:        Development files for GASNet
Group:          Development/Libraries/C and C++
Requires:       %{name} = %{version}
Requires:       lib%{name}-%{libver}

%description devel
GASNet is a language-independent, low-level networking layer that provides
network-independent communication primitives tailored for
implementing parallel global address space SPMD languages
such as UPC, Titanium, and Co-Array Fortran.

The interface is primarily intended as a compilation target and for
use by runtime library writers (as opposed to end users).

Development package for GASNet. Including header files and libraries.

%package -n libgasnet-%{libver}
Summary:        Runtime libraries for GASNet
Group:          System/Libraries

%description -n libgasnet-%{libver}
GASNet is a language-independent, low-level networking layer that provides
network-independent communication primitives tailored for
implementing parallel global address space SPMD languages
such as UPC, Titanium, and Co-Array Fortran.

This package contains the libraries for GASNet.

%package doc
Summary:        Documentation for GASNet
Group:          Documentation/Other
BuildArch:      noarch

%description doc
GASNet is a language-independent, low-level networking layer that provides
network-independent communication primitives tailored for
implementing parallel global address space SPMD languages
such as UPC, Titanium, and Co-Array Fortran.

This package contains the documentation for GASNet.

%prep
%setup -q -n GASNet-%{version}

# Avoid unnecessary rebuilds of the package
FAKE_BUILDDATE=$(LC_ALL=C date -u -r %{_sourcedir}/%{name}.changes '+%%b %%e %%Y')
FAKE_BUILDTIME=$(LC_ALL=C date -u -r %{_sourcedir}/%{name}.changes '+%%H:%%M:%%S')
sed -i -e "s/__DATE__/\"$FAKE_BUILDDATE\"/" -e "s/__TIME__/\"$FAKE_BUILDTIME\"/" gasnet_tools.c gasnet_trace.c tests/test.h

%build
%configure --enable-udp --enable-mpi --enable-par --enable-ibv --disable-aligned-segments  --enable-segment-fast --with-segment-mmap-max=4GB CC="gcc -fPIC" CXX="g++ -fPIC"
make %{?_smp_mflags} MANUAL_CFLAGS="%optflags -fPIC" MANUAL_MPICFLAGS="%optflags -fPIC" MANUAL_CXXFLAGS="%optflags -fPIC"

%check
make %{?_smp_mflags} check MANUAL_CFLAGS="%optflags -fPIC" MANUAL_MPICFLAGS="%optflags -fPIC" MANUAL_CXXFLAGS="%optflags -fPIC"

%install
make %{?_smp_mflags} DESTDIR=%{buildroot} install
%fdupes %{buildroot}/%{_prefix}
chmod +x %{buildroot}/%{_bindir}/*.pl

#Upstream doesn't want to support shared libs: https://bitbucket.org/berkeleylab/gasnet/pull-requests/36
#mind the order for link deps, libgasnet-smp-par first then libam* then the rest
for l in %{buildroot}/%{_libdir}/lib{gasnet-smp-par,am*,*}.a; do \
    [[ -f $l ]] || continue; \
    soname=`basename $l .a`; \
    libdir=`dirname $l`; \
	linker=`which g++`
    libs= ; \
    [[ ${soname} = libgasnet-*-par* ]] && libs+=" -lpthread"; \
    [[ ${soname} = libgasnet-psm-seq ]] && libs+=" -lpthread"; \
    [[ ${soname} = libamudp ]] && libs+=" -L${libdir} -lgasnet-smp-par"; \
    [[ ${soname} = libammpi ]] && libs+=" $(mpicc --showme:link)"; \
    [[ ${soname} = libgasnet-udp-* ]] && libs+=" -L${libdir} -lamudp"; \
    [[ ${soname} = libgasnet-mpi-* ]] && libs+=" -L${libdir} -lammpi"; \
    [[ ${soname} = libgasnet-ibv-* ]] && libs+=" -L${libdir} -libverbs -lpthread" &&
          linker=`which mpic++`; \
    [[ ${soname} = libgasnet-psm-* ]] && libs+=" -L${libdir} -lpsm2" &&
	   linker=`which mpic++`; \
    [[ ${soname} = libgasnet-ofi-* ]] && libs+=" -L${libdir} -lfabric" &&
	   linker=`which mpic++`; \
    [[ ${soname} = libgasnet-*-* ]] && libs+=" -lrt"; \
    ${linker} -shared -Wl,-soname=${soname}-%{version}.so \
        -Wl,--as-needed -Wl,-z,defs -Wl,--rpath-link=. \
        -Wl,--whole-archive ${l} -Wl,--no-whole-archive \
        ${libs} -o ${libdir}/${soname}-%{version}.so && \
    ln -s ${soname}-%{version}.so ${libdir}/${soname}.so && \
    rm ${l} ; \
done

%post -n libgasnet-%{libver} -p /sbin/ldconfig
%postun -n libgasnet-%{libver} -p /sbin/ldconfig

%files
%defattr(-,root,root)
%{_bindir}/*

%files -n libgasnet-%{libver}
%defattr(-,root,root)
%{_libdir}/lib*-%{version}.so

%files doc
%defattr(-,root,root)
%{_datadir}/doc/GASNet

%files devel
%defattr(-,root,root)
%{_includedir}/*
%{_libdir}/lib*[a-z].so
%{_libdir}/pkgconfig/*.pc
%{_libdir}/valgrind

%changelog
* Wed May 31 2017 nmoreychaisemartin@suse.com
- Build gasnet for all arch but s390*, s390x not supported by upstream:
  https://upc-bugs.lbl.gov/bugzilla/show_bug.cgi?id=3523
- PSM2 support is only for x86_64
* Tue May 16 2017 nmoreychaisemartin@suse.com
- Support gasnet over OFI and ibverbs
* Fri Apr  7 2017 pth@suse.de
- Remove conflict marker.
* Fri Mar 31 2017 nmoreychaisemartin@suse.com
- Fix dynamic linking for PSM libraries
* Fri Mar 31 2017 pth@suse.de
- GASnet relies on libpsm2-compat which again only supports x86_64
  so make GASnet do the same.
* Wed Mar 29 2017 pth@suse.de
- Make dependencies on libs now coming from rdma-core versioned.
* Sat Mar 18 2017 junghans@votca.org
- Version update to 1.28.2, lot of changes, see:
  https://gasnet.lbl.gov/dist/ChangeLog
* Mon Feb  6 2017 jengelh@inai.de
- Declare bash as build shell
* Mon Oct 24 2016 junghans@votca.org
- Version update to 1.28.0, lot of changes, see:
  https://gasnet.lbl.gov/dist/ChangeLog
* Mon Oct 17 2016 jengelh@inai.de
- Implement shared library naming guideline
- Trim description
* Tue Sep 20 2016 junghans@votca.org
- added %%check block and related patch from upstream:
  https://bitbucket.org/berkeleylab/gasnet/pull-requests/37
- link shared libs, so that deps are fulfilled
* Mon Sep 12 2016 junghans@votca.org
- Added upstream patch to support overwriting of flags:
  https://bitbucket.org/berkeleylab/gasnet/pull-requests/34
- Build shared libs manually as upstream doesn't want to support them:
  https://bitbucket.org/berkeleylab/gasnet/pull-requests/36
  but we need them for legion package
* Sun Sep  4 2016 junghans@votca.org
- Update to GASNet 1.26.4
  Details: https://gasnet.lbl.gov/dist/ChangeLog
* Sun May 29 2016 junghans@votca.org
- import into Science overlay
* Thu Mar 10 2016 nirmoy.das@suse.com
- Initial package, version: 1.26.0(fate#319250)
